import shutil
from threading import local
from .plugin import JobServerPlugin
import os
import json
import sys
from slugify import slugify

# the libgenders module is used to generate a genders file for use 
# with the pdsh command.

class libgenders(JobServerPlugin):
  jobModule = 'libgenders'
  gendersFile = "/etc/genders"
  def handle_jobs(self):
    nodes = None
    images = None
    groups = None
    distros = None
    # Update all pending jobs of oUr module to running
    if(not self.set_job_running()):
      # no jobs to run, just return
      return
    print("[libgenders] Handling Job")
    # get a list of all nodes.
    response = self.js.session.get( self.js.mprovURL + 'systems/')
    if not self.checkHTTPStatus(response.status_code):
      print("Error[libgenders]: Unable to get a list of sysetms from the mPCC.")
      self.set_job_failure()
      return
    nodes = response.json()
    # get a list of all groups. 
    response = self.js.session.get( self.js.mprovURL + 'systemgroups/')
    if not self.checkHTTPStatus(response.status_code):
      print("Error[libgenders]: Unable to get a list of system groups from the mPCC.")
      self.set_job_failure()
      return
    groups = response.json()

    # get a list of all the images.
    response = self.js.session.get ( self.js.mprovURL + 'systemimages/')
    if not self.checkHTTPStatus(response.status_code):
      print("Error[libgenders]: Unable to get a list of system images from the mPCC.")
      self.set_job_failure()
      return    
    images = response.json()

    # get a list of all os distros.
    response = self.js.session.get ( self.js.mprovURL + 'distros/')
    if not self.checkHTTPStatus(response.status_code):
      print("Error[libgenders]: Unable to get a list of OS Distrubutions from the mPCC.")
      self.set_job_failure()
      return    
    distros = response.json()

    # loop through all nodes and add them to the genders file making 
    # sure to add all system groups and os's they are part of.
    genders_file_contents = """
# This genders file is maintained by mProv.
# if you would like entries included in this file
# create an /etc/genders.include file and put them 
# there.  They will be added to the end of this file.
# All libgenders rules for this file and any 
# included files still apply.
#
# See 'man libgenders' for more information.
#
# THIS FILE WILL BE OVERRIDDEN ON NODE SAVES!
# DO NOT DIRECTLY MODIFY THIS FILE AND EXPECT YOUR
# CHANGES TO BE SAVED
# """
    try:
      with open(self.gendersFile, 'w') as gendersFile:
        gendersFile.write(genders_file_contents + "\n")
        for node in nodes:
          try:
            image_name = node['systemimage']
            # find the system image in question
            image = [image for image in images if image.get('slug')==image_name]
            image = image[0]
            
            # find the distro for the image
            distro = [distro for distro in distros if distro.get('id')==image['osdistro']['id']]
            distro = distro[0]
            node_groups = ""
            for groupId in node['systemgroups']:
              group = [group for group in groups if group.get('id')==groupId]
              group = group[0]
              node_groups += group['name']
            line = f"{node['hostname']}   os={slugify(distro['name'])},image={image_name},{node_groups},all"
          except Exception as f:
            exc_type, exc_obj, exc_tb = sys.exc_info()

            line = f"# Error processing node {node['hostname']}: {f} Line: {exc_tb.tb_lineno}"
          gendersFile.write(line + "\n")
    except Exception as e:
        print(f"Error[libgenders]: Unable to write to {self.gendersFile}: {e}")
        self.set_job_failure()
        return
    self.set_job_success()
    print("[libgenders] Job Complete.")






import requests
import socket
import uuid
import yaml, os, sys, glob, time
class MProvScript():
  '''
  This class is a template that can be used by scripts run
  by the 'script-runner' to have access to the jobserver 
  config to run against the API.
  '''
  config_data = {}
  mprovURL = "http://127.0.0.1:8080/"
  apikey = ""
  heartbeatInterval = 10
  runonce = True
  sessionOk = False
  disklayout = {}
  session = requests.Session()
  ip_address = None
  configfile = "/etc/mprov/script-runner.yaml"
  def __init__(self, **kwargs):
    # load our config
    self.load_config()

    # start a session
    if not self.startSession():
      print("Error: Unable to communicate with the mPCC.")
      sys.exit(1)

  def yaml_include(self, loader, node):
    # disable includes, no need.
    return {}

  def load_config(self):
    # load the config yaml
    # print(self.configfile)
    yaml.add_constructor("!include", self.yaml_include)
    
    if not(os.path.isfile(self.configfile) and os.access(self.configfile, os.R_OK)):
      print("Error: Unable to find a working config file.")
      sys.exit(1)

    with open(self.configfile, "r") as yamlfile:
      self.config_data = yaml.load(yamlfile, Loader=yaml.FullLoader)

    # flatten the config space
    result = {}
    for entry in self.config_data:
      result.update(entry)
    self.config_data = result

    # map the global config on to our object
    for config_entry in self.config_data['global'].keys():
      try:
        getattr(self, config_entry)
        setattr(self, config_entry, self.config_data['global'][config_entry])
      except:
        # ignore unused keys
        pass
    pass

  def startSession(self):
    
    self.session.headers.update({
      'Authorization': 'Api-Key ' + self.apikey,
      'Content-Type': 'application/json'
      })

    # connect to the mPCC
    try:
      response = self.session.get(self.mprovURL, stream=True)
    except:
      print("Error: Communication error to the server.  Retrying.", file=sys.stderr)
      self.sessionOk = False
      time.sleep(self.heartbeatInterval)
      return False
    self.sessionOk = True
    # get the sock from the session
    s = socket.fromfd(response.raw.fileno(), socket.AF_INET, socket.SOCK_STREAM)
    # get the address from the socket
    address = s.getsockname()
    self.ip_address=address
      
    # if we get a response.status_code == 200, we're ok.  If not,
    # our auth failed.
    return response.status_code == 200

  # OVERRIDE THIS!
  def run(self):
    pass

  def main(self):
    self.startSession()
    self.run()



from jinja2 import Environment, PackageLoader, select_autoescape
from mprov_jobserver.plugins.plugin import JobServerPlugin
import os, socket, ipaddress, netifaces
from socket import AF_INET, AF_INET6


jenv = Environment(
    loader=PackageLoader("mprov_jobserver"),
    autoescape=select_autoescape()
)

class DnsmasqDNSConfig(JobServerPlugin):
    dnsmasqConfDir=''
    mprovDnsmasqDir=''
    hostname=''
    def __init__(self, js):
        super().__init__(js)
        self.hostname = socket.gethostname()
        if '.' in self.hostname:
            self.hostname, _ = self.hostname.split('.', 1)
    def load_config(self):
        return True
    def _inmProvNet(self, ipaddr, net):
        iface_ip = ipaddress.ip_address(ipaddr)
        iface_net = ipaddress.ip_network(f"{net['subnet']}/{net['netmask']}", strict=False)
        if iface_ip in iface_net:
            return True
        return False
    def _addSelfToNet(self, network):
        selfHosts = []
        for iface in netifaces.interfaces():
            # do not run on the local interface.
            if iface == 'lo':
                continue

            iface_details = netifaces.ifaddresses(iface)
            # only process IPv4 and IPv6 addresses.
            if AF_INET in iface_details or AF_INET6 in iface_details:
                for af in [AF_INET, AF_INET6]:
                    if af in iface_details:
                        for address in iface_details[af]:
                            if "%" in address['addr']:
                                address['addr'], _ = address['addr'].split('%', 1)
                            if self._inmProvNet(address['addr'], network):
                                selfHosts.append({
                                    'ipaddress': address['addr'],
                                    'ipv6ll': '',
                                    'hostname': self.hostname,
                                    'domain': network['domain'] + f"# {iface}"
                                })
            # # grab and add the LL ipv6 ip addresses.
            # if AF_INET6 in netifaces.ifaddresses(iface):
            #     for v6ip in netifaces.ifaddresses(iface)[AF_INET6]:
            #         if v6ip['addr'].startswith("fe80:"):
            #             if "%" in v6ip['addr']:
            #                 v6ip['addr'], _ = v6ip['addr'].split("%", 1)
            #             selfHosts.append({
            #                 'ipaddresss': '',
            #                 'ipv6ll': v6ip['addr'],
            #                 'hostname': self.hostname,
            #                 'domain': network['domain'] + f"# {iface}"
            #             })
                


        return selfHosts
               
    def handle_jobs(self):

        # get the network informatioin
        response = self.js.session.get( self.js.mprovURL + 'networks/?managedns=True')
        data = {
            'networks': response.json(),
            'enableDNS': True,
        }
        with open(self.dnsmasqConfDir + 'dns.conf', 'w') as conf:
            conf.write(jenv.get_template('dnsmasq/dns.conf.j2').render(data))

        # make sure the mprov hosts dir exists
        os.makedirs(self.mprovDnsmasqDir + '/dns/', exist_ok=True)
        # now the interfaces
        for network in data['networks']: 
            
        
            # grab the system network interfaces that are on this network.
            response = None
            data_hosts = { 'hosts': [] }
            print(self.js.mprovURL + 'networkinterfaces/?network=' + network['slug'])
            response = self.js.session.get( self.js.mprovURL + 'networkinterfaces/?network=' + network['slug'])
            if self.checkHTTPStatus(response.status_code):
                data_hosts = {
                    'enableDNS': True,
                    'hosts': response.json(),
                }
            # add ourself if we are listening on this net
            data_hosts['hosts'].extend(self._addSelfToNet(network))

            # grab the bmcs that are on this network.
            print(self.js.mprovURL + f"systembmcs/?network={network['id']}&detail")
            response = self.js.session.get(self.js.mprovURL + f"systembmcs/?network={network['id']}&detail")
            if self.checkHTTPStatus(response.status_code):
                # only do the bmcs if we get some.
                
                try: 
                    for bmc in response.json():
                        tmpHost = {
                            'ipaddress': bmc['ipaddress'],
                            'ipv6ll': "", 
                            'mac': bmc['mac'],
                            'hostname': bmc['system']['hostname'],
                            'domain': network['domain']
                        }
                        data_hosts['hosts'].append(tmpHost)
                except:
                    pass
            # merge in the switches
            response = self.js.session.get( self.js.mprovURL + 'switches/?network=' + network['slug'])
            data_hosts['hosts'] = data_hosts['hosts'] + (response.json())
            
            # print(data_hosts)
            for idx, host in enumerate(data_hosts['hosts']):
                if 'mgmt_mac' in host:
                    data_hosts['hosts'][idx]['mac'] = host['mgmt_mac']
                if 'mgmt_ip' in host:
                    data_hosts['hosts'][idx]['ipaddress'] = host['mgmt_ip']
                data_hosts['hosts'][idx]['domain'] = network['domain']
                # print(data_hosts['hosts'][idx])

            with open(self.mprovDnsmasqDir + '/dns/' + network['slug'] + '-dns.conf', 'w') as conf:
                conf.write(jenv.get_template('dnsmasq/dns_host.conf.j2').render(data_hosts))
        # restart dnsmasq
        os.system('systemctl restart dnsmasq')




from distutils.command.build import build
from .dnsmasq_mod.config import DnsmasqConfig
from .dnsmasq_mod.dns import DnsmasqDNSConfig
from .dnsmasq_mod.dhcp import DnsmasqDHCPConfig
from .plugin import JobServerPlugin
import sh, os, sys

# class to handle dnsmaq generation.
# This class can job modules:
# - pxe-update
# - pxe-delete
# - dns-update
# - dns-delete
# - dhcp-update
# - dhcp-delete
# If other jobmodules are running that can consume these jobmodules,
# you may have unexpected results.
class dnsmasq(JobServerPlugin):
  jobModule = 'dnsmasq'
  enableDNS=False
  enableDHCP=False
  enableTFTP=False
  dnsmasqConfDir=''
  mprovDnsmasqDir=''
  tftproot=''
  undionlyImg = ''
  dnsmasqUser=''
  threads = []
  ipxe_files = ['undionly.kpxe', 'snponly.efi', 'snponly_ipv4.efi']
  
# dnf -y install mkisofs
# cd /tmp
# git clone https://github.com/ipxe/ipxe
# cd ipxe/src
# make
# /bin/cp -f bin/undionly.kpxe /tftpboot 
# make bin-x86_64-efi/snponly.efi
# /bin/cp -f bin-x86_64-efi/snponly.efi /tftpboot/snponly.efi

  def load_config(self):
    result =  super().load_config()
    if result:
      os.makedirs(f"{self.tftproot}", exist_ok=True)
      # Check for the ipxe files in the tftproot, and build it from 
      # git source if not there.
      buildIpxe=False
      for file in self.ipxe_files:
        if not os.path.exists(f"{self.tftproot}/{file}"):
          buildIpxe = True

      if buildIpxe:
        print("Rebuilding iPXE files, some or all are missing.")
        sh.dnf(['-y', 'install', 'mkisofs'])
        oldCwd = os.getcwd()
        os.chdir('/tmp')
        print("\tCheck out iPXE code to /tmp/ipxe/")
        try:
          sh.rm('-rf', '/tmp/ipxe')
          sh.git(['clone', 'https://github.com/ipxe/ipxe'])
        except:
          print("Error: unable to fetch ipxe source.  You may need to copy some files yourself.")
          return result
        
        print("\tRunning inintial build")
        os.chdir('ipxe/src')
        try:
          sh.make(['-j4', 'bin-x86_64-efi/snponly.efi', 'bin/undionly.kpxe'])
        except:
            print("Error: iPXE make command failed.")
            return result
        try:
          sh.cp(['-f','bin/undionly.kpxe', f"{self.tftproot}"])
        except:
          print(f"Error: Unable to copy bin/undionly.kpxe -> {self.tftproot}")
        try:
           sh.cp(['-f','bin-x86_64-efi/snponly.efi', f"{self.tftproot}"])
        except:
          print(f"Error: Unable to copy bin-x86_64-efi/snponly.efi -> {self.tftproot}")
        
        # now we will rebuild the EFI version with IPv6 support turned off
        print("\tPatching EFI to IPv4 only...")
        with open("config/defaults/efi.h", "r") as f:
            lines = f.readlines()
        with open("config/defaults/efi.h", "w") as f:
            for line in lines:
                if "NET_PROTO_IPV6" in line.strip("\n"):
                    f.write(f"//{line}")
                else:
                  f.write(f"{line}")
        print("\tRunning second build...")
        try:
          sh.make(['-j4','bin-x86_64-efi/snponly.efi'], _out=sys.stdout, _err=sys.stderr)
        except:
            print("Error: iPXE IPv4 only EFI make command failed.")
            sh.touch([f"{self.tftproot}/snponly_ipv4.efi"])
            sys.exit(1) #return result
        try:
           sh.cp(['-f','bin-x86_64-efi/snponly.efi', f"{self.tftproot}/snponly_ipv4.efi"])
        except:
          print(f"Error: Unable to copy bin-x86_64-efi/snponly.efi -> {self.tftproot}/snponly_ipv4.efi")
        print("Build Process finished.  Errors appear above.")
        os.chdir(oldCwd)

      pass
    return result
  def handle_jobs(self):
    # we can also run the DNS thread
    if self.enableDNS:
      # grab any DNS jobs.
      # See if we have any image-delete jobs, and take 'em if we do, else just exit
      jobquery = "&module=[\"dns-update\",\"dns-delete\"]"
      # print(jobquery)
      if not self.js.update_job_status(self.jobModule, 2, jobquery=jobquery + "&status=1"):
        pass # no jobs.
        self.enableDNS = False
      else:
        dnsThread = DnsmasqDNSConfig(self.js)
        dnsThread.dnsmasqConfDir = self.dnsmasqConfDir
        dnsThread.mprovDnsmasqDir = self.mprovDnsmasqDir
        dnsThread.start()
        self.threads.append(dnsThread)

    # and finally the DHCP thread
    if self.enableDHCP:
      # grab any DHCP/PXE jobs.
      # See if we have any image-delete jobs, and take 'em if we do, else just exit
      jobquery = "&module=[\"pxe-update\",\"dhcp-update\",\"pxe-delete\",\"dhcp-delete\"]"
      # print(jobquery)
      if not self.js.update_job_status(self.jobModule, 2, jobquery=jobquery + "&status=1"):
        pass # no jobs.
        self.enableDHCP = False
      else:
        dhcpThread = DnsmasqDHCPConfig(self.js)
        dhcpThread.dnsmasqConfDir = self.dnsmasqConfDir
        dhcpThread.mprovDnsmasqDir = self.mprovDnsmasqDir
        dhcpThread.tftproot=self.tftproot
        
        dhcpThread.enableTFTP = self.enableTFTP
        dhcpThread.start()
        self.threads.append(dhcpThread)
    # Based on our settings, let's start up the submodules for dnsmasq.
    if(self.enableDNS or self.enableDHCP):
      
      # DNS or DHCP is on, so let's run the config module
      confThread = DnsmasqConfig(self.js)
      confThread.dnsmasqConfDir=self.dnsmasqConfDir
      confThread.mprovDnsmasqDir = self.mprovDnsmasqDir
      confThread.tftproot = self.tftproot
      confThread.dnsmasqUser = self.dnsmasqUser
      confThread.start()
      self.threads.append(confThread)    

    for thread in self.threads:
      thread.join()
      


from ipaddress import IPv4Network
import ipaddress
import os
from .plugin import JobServerPlugin
import time
import json
from glob import glob

"""
Node Auto Detection System

Parts for LLDP parsing are borrowed from https://github.com/GoozeyX/lldp.discovery

"""
import netifaces
import fcntl, struct

from ctypes import c_char, c_short, Structure
from fcntl import ioctl
from socket import socket, htons, inet_ntoa
from socket import AF_PACKET, AF_INET, SOCK_DGRAM, SOCK_RAW
from socket import gaierror

from struct import pack, unpack

from dmidecode import DMIDecode
from pyroute2 import IPDB

## Magic constants from `/usr/include/linux/if_ether.h`:
ETH_P_ALL = 0x0003
ETH_ALEN = 6
ETH_HLEN = 14

## LLDP Ethernet Protocol:
# LLDP Length:
LLDP_TLV_TYPE_BIT_LEN = 7
LLDP_TLV_LEN_BIT_LEN = 9
LLDP_TLV_HEADER_LEN = 2         # 7 + 9 = 16
LLDP_TLV_OUI_LEN = 3
LLDP_TLV_SUBTYPE_LEN = 1
# LLDP Protocol BitFiddling Mask:
LLDP_TLV_TYPE_MASK = 0xfe00
LLDP_TLV_LEN_MASK = 0x1ff
# LLDP Protocol ID:
LLDP_PROTO_ID = 0x88cc
# LLDP TLV Type:
LLDP_TLV_TYPE_CHASSISID = 0x01
LLDP_TLV_TYPE_PORTID = 0x02
LLDP_TLV_TYPE_PORTDESC = 0x04
LLDP_TLV_DEVICE_NAME = 0x05
LLDP_PDUEND = 0x00
LLDP_TLV_ORGANIZATIONALLY_SPECIFIC = 0x7f
# LLDP TLV OUI Type:
LLDP_TLV_OUI_802_1 = 0x0008c2
LLDP_TLV_OUI_802_3 = 0x00120f

## Magic string for unpack packet:
UNPACK_ETH_HEADER_DEST = '!%s' % ('B' * ETH_ALEN)
UNPACK_ETH_HEADER_SRC = '!%s' % ('B' * ETH_ALEN)
UNPACK_ETH_HEADER_PROTO = '!H'

## Magic string for unpack LLDP packet:
UNPACK_LLDP_TLV_TYPE = '!H'
UNPACK_LLDP_TLV_OUI = '!%s' % ('B' * LLDP_TLV_OUI_LEN)
UNPACK_LLDP_TLV_SUBTYPE = '!B'

## Other info about network under linux:
NETDEV_INFO = '/proc/net/dev'
SIOCGIFADDR = 0x8915    # Socket opt for get ip addr under linux
SIOCSIFHWADDR = 0x8927  # Socket opt for get mac addr under linux
SIOCGIFFLAGS = 0x8913   # `G` for Get socket flags
SIOCSIFFLAGS = 0x8914   # `S` for Set socket flags
IFF_PROMISC = 0x100     # Enter Promiscuous mode

def promiscuous_mode(interface, sock, enable=False):
    """ Enable/Disable NIC promiscuous mode via `ioctl` system call
            with c-compatible `ifreq` struct and `SIOC[G|S]IFFLAGS` """

    ifr = ifreq()
    ifr.ifr_ifrn = bytes(interface, 'utf-8')
    ioctl(sock.fileno(), SIOCGIFFLAGS, ifr)

    if enable:
        ifr.ifr_flags |= IFF_PROMISC
    else:
        ifr.ifr_flags &= ~IFF_PROMISC
    ioctl(sock.fileno(), SIOCSIFFLAGS, ifr)


def unpack_ethernet_frame(packet):
    """ Unpack ethernet frame """

    eth_header = packet[0:ETH_HLEN]
    eth_dest_mac = unpack(UNPACK_ETH_HEADER_DEST, eth_header[0:ETH_ALEN])
    eth_src_mac = unpack(UNPACK_ETH_HEADER_SRC, eth_header[ETH_ALEN:ETH_ALEN*2])
    eth_protocol = unpack(UNPACK_ETH_HEADER_PROTO, eth_header[ETH_ALEN*2:ETH_HLEN])[0]
    eth_payload = packet[ETH_HLEN:]

    return (eth_header, eth_dest_mac, eth_src_mac, eth_protocol, eth_payload)


def covert_hex_string(decimals):
    """ Covert decimals to hex string which start with `0x`, 
            and `strip` by `0x` """
    return [ hex(decimal).strip('0x').rjust(2, '0') for decimal in decimals ]


def unpack_lldp_frame(eth_payload):
    """ Unpack lldp frame """

    while eth_payload:

        tlv_header = unpack(UNPACK_LLDP_TLV_TYPE, eth_payload[:LLDP_TLV_HEADER_LEN])
        tlv_type = (tlv_header[0] & LLDP_TLV_TYPE_MASK) >> LLDP_TLV_LEN_BIT_LEN
        tlv_data_len = (tlv_header[0] & LLDP_TLV_LEN_MASK)
        tlv_payload = eth_payload[LLDP_TLV_HEADER_LEN:LLDP_TLV_HEADER_LEN + tlv_data_len]

        # These headers only available with 
        #   `LLDP_TLV_ORGANIZATIONALLY_SPECIFIC` TLV
        tlv_oui = None
        tlv_subtype = None

        if tlv_type == LLDP_TLV_ORGANIZATIONALLY_SPECIFIC:
            _tlv_oui = unpack(UNPACK_LLDP_TLV_OUI, tlv_payload[:LLDP_TLV_OUI_LEN])
            tlv_subtype = unpack(UNPACK_LLDP_TLV_SUBTYPE, 
                            tlv_payload[LLDP_TLV_OUI_LEN:LLDP_TLV_OUI_LEN + LLDP_TLV_SUBTYPE_LEN])[0]
            tlv_payload = tlv_payload[LLDP_TLV_OUI_LEN + LLDP_TLV_SUBTYPE_LEN:]
                
            # Covert oui from list to hex/decimals
            tlv_oui = str()
            for bit in _tlv_oui:
                tlv_oui += hex(bit).strip('0x').rjust(2, '0')
            tlv_oui = int(tlv_oui, 16)

        elif tlv_type == LLDP_PDUEND:
            break

        eth_payload = eth_payload[LLDP_TLV_HEADER_LEN + tlv_data_len:]

        yield (tlv_header, tlv_type, tlv_data_len, tlv_oui, \
                                        tlv_subtype, tlv_payload)
    


class ifreq(Structure):
    """ C-compatible `ifreq` struct """
    _fields_ = [("ifr_ifrn", c_char * 16),
                ("ifr_flags", c_short)]


class nads(JobServerPlugin):
  jobModule = 'nads'
  provIntf = ''
  maxLLDPWait = 60
  port = None
  switch = None
  mac = None
  reboot = False

  def getLLDP(self):
    startTime = time.time()

    # setup a capture socket.
    capture_sock = socket(AF_PACKET, SOCK_RAW, htons(ETH_P_ALL))

    # enable promiscuous mode on the interface. 
    promiscuous_mode(self.provIntf, capture_sock, True)
    print("Start at: " + str(startTime))
    # grab some traffic and process it for LLDP
    while(time.time() < (startTime + self.maxLLDPWait)):
      # print("Current: " + str(time.time()))
      # print("End at: " + str(startTime + self.maxLLDPWait))
      # grab a packet.
      packet = capture_sock.recvfrom(65565)
      packet = packet[0]
      eth_protocol, eth_payload = unpack_ethernet_frame(packet)[3:]

      if eth_protocol == LLDP_PROTO_ID:
        # turn off promiscuous mode while we process our packet.  
        # XXX: Should we wait until we are done to turn this off?
        promiscuous_mode(self.provIntf, capture_sock, False)

        for tlv_parse_rv in unpack_lldp_frame(eth_payload):
  
          tlv_header, tlv_type, tlv_data_len, tlv_oui, tlv_subtype, tlv_payload \
                                                                  = tlv_parse_rv

          # if tlv_type == LLDP_TLV_TYPE_PORTDESC:
          #   self.port = tlv_payload.decode('utf-8')
          if tlv_type == LLDP_TLV_DEVICE_NAME:
            self.switch= tlv_payload.decode('utf-8')
          elif tlv_type == LLDP_TLV_TYPE_PORTID:
            self.port = tlv_payload.decode('utf-8')

          # exit our loops, we have what we came for.
          if  self.port is not None and self.switch is not None:
            # print("Exiting for loop: '" + self.port + "' and '" + self.switch + "'")
            break
      #time.sleep(1)
      if  self.port is not None and self.switch is not None:
        # print("Exiting while loop: '" + self.port + "' and '" + self.switch + "'")
        break
    # print("Current: " + str(time.time()))

    if self.port is None or self.switch is None:
      # if we get here and don't have a port and a switch name,
      # we cannot continue.
      print("Error: Unable to detect switch and port via LLDP")
      return False
    # if we get here, we're all set.
    return True


  # def getMac(self):

    # s = socket(AF_INET, SOCK_DGRAM)
    # info = fcntl.ioctl(s.fileno(), 0x8927,  struct.pack('256s', bytes(self.provIntf, 'utf-8')))
    # # print(info[18:24])
    # # for byte in info[18:24]:
      
    # #   print(hex(byte))
    # return ':'.join(['%02x' % char for char in info[18:24]])

  
  def handle_jobs(self):
    # see if we are being called from a runonce command
    if not self.js.runonce:
      print("Error: script-runner must be run in a 'runonce' jobserver session.")
      return False
    
    # Because Intel.... turn off the card intercepting LLDP packets, if i40e is used.
    ifceCmdFiles = glob("/sys/kernel/debug/i40e/*/command" )
    for ifaceCmdFile in ifceCmdFiles:
      try:
        with open(ifaceCmdFile, 'w') as ifile:
          ifile.write("lldp stop\n")
      except:
        # ignore all errors here because we really don't care.
        pass
    lldpRes = False
    # Try all the interfaces, first one to give us an LLDP packet is our winner.
    # TODO: Maybe allow for  trying multiple interfaces even if an LLDP is detected
    # but registration fails?
    ipdb = IPDB()
    for iface in netifaces.interfaces():
      if iface == 'lo':
        continue
      # check if the interface is operationally up.
      if ipdb.interfaces[iface].operstate != "UP":
        continue
      print(f"Attempting LLDP Capture on {iface}")
      self.provIntf = iface
      try:
        lldpRes = self.getLLDP()
      except Exception as e:
        print(f"{e}")
        lldpRes = False
      if lldpRes:
        # grab the mac of the prov interface.
        self.mac = netifaces.ifaddresses(self.provIntf)[AF_PACKET][0]['addr']
        break
    ipdb.release()
    # and now, we are supposed to turn it backon again.....
    for ifaceCmdFile in ifceCmdFiles:
      try:
        with open(ifaceCmdFile, 'w') as ifile:
          ifile.write("lldp start\n")
      except:
        # ignore all errors here because we really don't care.
        pass
    
    if lldpRes == False:

      return False

    

    if self.port.find('/') >= 0:
      # we need to parse the port out of the port id.
      if self.port.rfind('/') >= 0:
        self.port = self.port[self.port.rfind('/')+1:]

    # if we got LLDP, tell the MPCC who we are, or better put, send it our switch, port, and mac.
    # and let it sort it out.
    query = '/systems/register'

    # grab our DMIDecode information
    dmi = DMIDecode()
    data = {
      'switch': self.switch,
      'port': self.port,
      'vendor': dmi.manufacturer(),
      'model': dmi.model(),
      'mac': self.mac,
    }
    print("Attempting to register with MPCC...")
    print(data)
    response = self.js.session.post(f"{self.js.mprovURL}{query}", data=json.dumps(data))
    if response.status_code == 200:
      print("We were able to register.")
      # Issue  a reboot if we are supposed to.
      if self.reboot:
        os.system("/sbin/reboot")
      return True
    print("There was a problem registering with the MPCC.")

    return False